#! /usr/bin/python
# -*- coding: utf-8 -*-

## ui
import gtk
import webkit

## launch app
from subprocess import Popen

## python mod
import re
import os
import os.path
import sys

## xdg menu
import xdg.Menu
from xdg import BaseDirectory

#realpath = os.path.dirname(os.path.realpath( __file__ ))
realpath = os.path.dirname(os.environ['HOME'] + "/.config/html_desktop/")
if not os.path.exists(realpath):
	realpath = os.path.dirname("/usr/share/html_desktop/")
os.chdir(realpath)

## For test ( True/False )
DEBUG = 0


def launch_command(cmd):
		os.chdir(os.environ['HOME'])
		try:
			Popen(cmd, shell=True)
		except OSError:
			pass
		os.chdir(realpath)

def find_icon(icon):
	if not icon:
		icon = "default.png"
		return icon
		
	if not os.path.exists(icon):
		if re.match(".*\.(png|xpm|svg)$", icon) is not None:
			icon = icon[:-4]

		is_ok = False
		reps = ("icons/hicolor/48x48/apps", "icons/gnome/48x48/apps", 
				"icons/Humanity/apps/48", "icons/gnome-colors-common/scalable/apps", 
				"pixmaps", "icons/SliTaz/apps/22")
				
		for rep in reps:
			for ext in ('.png', '.svg'): #, '.xpm'):
				if os.path.isfile("/usr/share/" + rep + '/' + icon + ext):
					icon = "/usr/share/" + rep + '/' + icon + ext
					is_ok = True
					break
			if is_ok:
				break
		
		if not is_ok:
			icon = "default.png"

	return icon

class HTML_Desktop():

	def __init__(self):
		self.isVisible = False
		
		## Status Icon
		self.statusIcon = gtk.status_icon_new_from_file('ad.png')
		self.statusIcon.set_visible(True)
		self.statusIcon.set_tooltip('SliTaz HTML Desktop')
		self.statusIcon.connect('activate', self.toggle_hide)
		
		##  main window 
		self.window = gtk.Window()
		self.window.move(0,0)
		self.window.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DESKTOP)
		self.window.set_title("SliTaz HTML Desktop")
		self.window.set_name("SliTaz HTML Desktop")
		##  main window callback
		self.window.connect('destroy', self._quit)
		#~ self.window.add_events(gtk.gdk.FOCUS_CHANGE_MASK)
		#~ self.window.connect("focus-out-event", self.lost_focus)

		if DEBUG:
			self.window.resize(800,600)
		else:
			self.window.set_skip_taskbar_hint(True)
			self.window.set_skip_pager_hint(True)
			self.window.stick()
			self.window.set_decorated(False)
			self.window.set_keep_below(True)


		scrolled = gtk.ScrolledWindow()
		scrolled.show()
		scrolled.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
		
		## webview
		self.webview = webkit.WebView()
		screen_x , screen_y = gtk.gdk.screen_width(), gtk.gdk.screen_height()
		self.webview.set_size_request(screen_x,screen_y) 
		self.webview.show()
		scrolled.add(self.webview)
		self.window.add(scrolled)
		self.webview.open(realpath + '/index.html')	
		self.webview.connect("navigation-policy-decision-requested", self.nav_request)
		self.webview.connect('load-finished', self.onLoad)
		
		
		self.toggle_hide()

	def toggle_hide(self, widget=None, event=None):
		if self.isVisible:
			self.isVisible = False
			self.window.hide()
		else:
			self.isVisible = True
			self.window.show()

	def lost_focus(self, widget, event):
		if self.isVisible:
			self.toggle_hide()

	def append_html(self, el_id, html):
		msg = "$('#%s').append(\"%s\");" % (el_id, html.replace('"','\\"'))
		self.webview.execute_script(msg)
		
	def onLoad(self, webview, frame):

		xdgmenu = XdgMenu().getMenu()

		app_sort = []
		for category in xdgmenu:
			app_sort.append(category)
		app_sort.sort()
		
		for category in app_sort:
			if len(xdgmenu[category][1]) == 0:
				continue

			## Fix for SliTaz
			if os.path.exists("/usr/share/icons/SliTaz"):
				icon_path = "/usr/share/icons/SliTaz/categories/22/"
			else:
				icon_path = "/usr/share/icons/gnome/24x24/categories/"
				
			## Fix for Wine
			if category == 'Wine':
				icon = icon_path + "applications-other.png"
			else:
				icon = "%s%s.png" % (icon_path, xdgmenu[category][0])

			## fix '&' bug 
			fixed_cat = category.replace(' ', '')
			fixed_cat = fixed_cat.replace('&', '_')

			## Category
			html = "<div class=\"menu_item\" id=\"%s\">" % fixed_cat
			html += "<img src=\"%s\">" % icon
			html += "<h2>%s</h2>" % category
			html += "</div>"
			
			self.append_html('categories', html)
			
			## App in Category
			html = "<div class=\"cat_box\" id=\"div_%s\">" % fixed_cat
			html += "<h1>%s</h1>" % category
			html += '<div class="category_box">'

			for app in xdgmenu[category][1]:

				icon = find_icon(app.Icon)

				## Fix %[a-z] 
				action = ''
				path_parts = app.Exec.split()
				for i in xrange(len(path_parts)):
					if not path_parts[i][0] == '%':
						action += ' ' + path_parts[i]
					
				if app.Terminal:
					action = 'x-terminal-emulator -e %s' % action
				
				html += "<div class=\"entry\" onClick=\"window.location.href='exec:%s'\">" % action
				html += "<img src=\"%s\" alt=\"%s\">" % (icon, app.Icon)
				html += "<h2>%s</h2>" % app.Name
				html += "<h3>%s</h3>" % app.Comment
				html += "</div>"
					
			html += "</div><!-- category_box -->"
			html += "</div><!-- cat_box -->"
			self.append_html('category', html)
			
		self.webview.execute_script('InitEvent();')

	def unmap_event(self,widget,event):
		widget.set_skip_taskbar_hint(False)
	
	def map_event(self,widget,event):
		widget.set_skip_taskbar_hint(True)

	def nav_request(self, webview, frame, request, navigation_action, policy_decision):
		uri = request.get_uri()
		
		if uri.startswith("exec:"):
			action = uri.replace('exec:','')
			## FIXME!! space = %20 ( html )
			action = action.replace('%20', ' ')
			print('Exec = %s' % action)
			launch_command(action)
			
			## Need to return true (don't load uri)
			return True
			
	def _quit(self, widget=None):			
		gtk.main_quit()

	def main(self):
		gtk.main()

class App:
	Name = ''
	Icon = ''
	Exec = ''
	Terminal = False
	Comment = ''

class XdgMenu():
	
	def __init__(self, select_menu='applications'):
		
		self.applications = {}
		self.category = None

		if os.path.isfile('/etc/xdg/menus/%s.menu' % select_menu):
			xdgmenu = xdg.Menu.parse('/etc/xdg/menus/%s.menu' % select_menu)
			self.process_menu(xdgmenu)
		else:
			print ('Menu -', '/etc/xdg/menus/%s.menu' % select_menu)
			print ('-- not found')

	def getMenu(self):
		return self.applications

	def process_menu(self, menu):

		for entry in menu.getEntries():
			if isinstance(entry, xdg.Menu.Menu):
				name = entry.getName()
				icon = entry.getIcon()
				self.applications[name] = (icon,[])
				self.category = name
				self.process_menu(entry)

			elif isinstance(entry, xdg.Menu.MenuEntry):
				app = App()
				de = entry.DesktopEntry
				app.Name = de.getName()
				app.Icon = de.getIcon()
				app.Comment = de.getComment()
				app.Exec = de.getExec()
				app.Terminal = de.getTerminal()
				self.applications[self.category][1].append(app)

HTML_Desktop().main()
